==> vssh-0.1.0.crate: vssh-0.1.0/src/subcommands/connect.rs <==
use crate::api::ApiClient;
use rand::distributions::Alphanumeric;
use rand::{thread_rng, Rng};
use std::fs::{canonicalize, read_to_string, remove_file, OpenOptions};
use std::io::{ErrorKind, Write};
use std::iter;
use std::process::{exit, Command};

pub async fn connect<'a>(
    client: &ApiClient,
    role: &'a str,
    key: &'a str,
    server: &'a str,
    options: &'a str,
) {
    // Convert relative to absolute path and ensure exists
    let path = match canonicalize(key) {
        Ok(path) => path,
        Err(e) => {
            match e.kind() {
                ErrorKind::NotFound => println!("Private key '{}' does not exist", key),
                _ => println!("Failed to convert relative to absolute path: {}", e),
            }
            exit(1);
        }
    };

    // Read public key to file
    let contents = match read_to_string(format!("{}.pub", path.as_path().to_str().unwrap())) {
        Ok(contents) => contents,
        Err(e) => {
            println!(
                "Failed to read public key '{}': {}",
                path.as_path().to_str().unwrap(),
                e
            );
            exit(1);
        }
    };

    // Sign the public key
    let signed = match client.sign(role.to_string(), contents).await {
        Ok(signed) => signed,
        Err(e) => {
            println!("Failed to sign public key: {}", e);
            exit(1);
        }
    };

    // Create output file
    let name = random_string(16);
    let mut file = match OpenOptions::new()
        .write(true)
        .create(true)
        .open(format!("/tmp/{}", name))
    {
        Ok(file) => file,
        Err(e) => {
            match e.kind() {
                ErrorKind::PermissionDenied => {
                    println!("Cannot write signed public key: permission denied")
                }
                _ => println!("Failed to open output file: {}", e),
            };
            exit(1);
        }
    };

    // Write to file
    match file.write_all(signed.as_bytes()) {
        Ok(_) => {}
        Err(e) => {
            println!("Failed to write to output file: {}", e);
            exit(1);
        }
    }

    // Run command
    let mut child = match Command::new("ssh")
        .arg("-i")
        .arg(key)
        .arg("-i")
        .arg(format!("/tmp/{}", name))
        .arg(server)
        .args(options.split_whitespace().collect::<Vec<&str>>())
        .spawn()
    {
        Ok(child) => child,
        Err(e) => {
            println!("Failed to start ssh command: {}", e);
            exit(1);
        }
    };

    // Bring to foreground
    child.wait().expect("Failed to wait on child");

    // Remove signed certificate
    match remove_file(format!("/tmp/{}", name)) {
        Ok(_) => {}
        Err(e) => {
            match e.kind() {
                ErrorKind::PermissionDenied => {
                    println!("Cannot remove signed certificate: permission denied")
                }
                _ => println!("Failed to remove signed certificate: {}", e),
            };
            exit(1);
        }
    };
}

/// Generate a random string of alphanumeric characters
fn random_string(n: usize) -> String {
    iter::repeat(())
        .map(|()| thread_rng().sample(Alphanumeric))
        .take(n)
        .collect()
}

==> vssh-0.2.0.crate: vssh-0.2.0/src/subcommands/connect.rs <==
use crate::api::ApiClient;
use std::fs::{canonicalize, read_to_string, remove_file, OpenOptions};
use std::io::{ErrorKind, Write};
use std::process::{exit, Command};

pub async fn connect<'a>(
    client: &ApiClient,
    role: String,
    private_key: Option<String>,
    public_key: Option<String>,
    server: String,
    options: String,
) {
    // Use provided private key or default
    let private_key = if let Some(private_key) = private_key {
        private_key
    } else {
        String::from("~/.ssh/id_rsa")
    };

    // Use provided public key or default
    let public_key = if let Some(public_key) = public_key {
        public_key
    } else {
        format!("{}.pub", private_key)
    };

    // Ensure private key exists
    let private_path = ensure_exists(&private_key);

    // Ensure public key exists
    let public_path = ensure_exists(&public_key);

    // Read public key to file
    let contents = match read_to_string(&public_path) {
        Ok(contents) => contents,
        Err(e) => {
            println!(
                "Failed to read public key '{}': {}",
                public_path.as_path().to_str().unwrap(),
                e
            );
            exit(1);
        }
    };

    // Sign the public key
    let signed = match client.sign(role.to_string(), contents).await {
        Ok(signed) => signed,
        Err(e) => {
            println!("Failed to sign public key: {}", e);
            exit(1);
        }
    };

    // Create output file
    let name = crate::util::random_string(16);
    let mut file = match OpenOptions::new()
        .write(true)
        .create(true)
        .open(format!("/tmp/{}", name))
    {
        Ok(file) => file,
        Err(e) => {
            match e.kind() {
                ErrorKind::PermissionDenied => {
                    println!("Cannot write signed public key: permission denied")
                }
                _ => println!("Failed to open output file: {}", e),
            };
            exit(1);
        }
    };

    // Write to file
    match file.write_all(signed.as_bytes()) {
        Ok(_) => {}
        Err(e) => {
            println!("Failed to write to output file: {}", e);
            exit(1);
        }
    }

    // Run command
    let mut child = match Command::new("ssh")
        .arg("-i")
        .arg(private_path)
        .arg("-i")
        .arg(format!("/tmp/{}", name))
        .arg(server)
        .args(options.split_whitespace().collect::<Vec<&str>>())
        .spawn()
    {
        Ok(child) => child,
        Err(e) => {
            println!("Failed to start ssh command: {}", e);
            exit(1);
        }
    };

    // Bring to foreground
    child.wait().expect("Failed to wait on child");

    // Remove signed certificate
    match remove_file(format!("/tmp/{}", name)) {
        Ok(_) => {}
        Err(e) => {
            match e.kind() {
                ErrorKind::PermissionDenied => {
                    println!("Cannot remove signed certificate: permission denied")
                }
                _ => println!("Failed to remove signed certificate: {}", e),
            };
            exit(1);
        }
    };
}

/// Ensure a file exists and also convert it to an absolute path if it was not
fn ensure_exists(path: &str) -> std::path::PathBuf {
    match canonicalize(path) {
        Ok(path) => path,
        Err(e) => {
            match e.kind() {
                ErrorKind::NotFound => println!("File '{}' does not exist", path),
                _ => println!("Failed to convert relative to absolute path: {}", e),
            };
            exit(1);
        }
    }
}
